import { IVpc, ISecurityGroup, SubnetSelection } from 'aws-cdk-lib/aws-ec2';
import { ContainerImage } from 'aws-cdk-lib/aws-ecs';
import { DockerImageCode } from 'aws-cdk-lib/aws-lambda';
import { Construct } from 'constructs';
/**
 * Properties for the `TokenInjectableDockerBuilder` construct.
 */
export interface TokenInjectableDockerBuilderProps {
    /**
     * The path to the directory containing the Dockerfile or source code.
     */
    readonly path: string;
    /**
     * Build arguments to pass to the Docker build process.
     * These are transformed into `--build-arg` flags.
     * @example
     * {
     *   TOKEN: 'my-secret-token',
     *   ENV: 'production'
     * }
     */
    readonly buildArgs?: {
        [key: string]: string;
    };
    /**
     * The ARN of the AWS Secrets Manager secret containing Docker login credentials.
     * This secret should store a JSON object with the following structure:
     * ```json
     * {
     *   "username": "my-docker-username",
     *   "password": "my-docker-password"
     * }
     * ```
     * If not provided, the construct will skip Docker login during the build process.
     *
     * @example 'arn:aws:secretsmanager:us-east-1:123456789012:secret:DockerLoginSecret'
     */
    readonly dockerLoginSecretArn?: string;
    /**
     * The VPC in which the CodeBuild project will be deployed.
     * If provided, the CodeBuild project will be launched within the specified VPC.
     * @default No VPC is attached, and the CodeBuild project will use public internet.
     */
    readonly vpc?: IVpc;
    /**
     * The security groups to attach to the CodeBuild project.
     * These should define the network access rules for the CodeBuild project.
     * @default No security groups are attached.
     */
    readonly securityGroups?: ISecurityGroup[];
    /**
     * The subnet selection to specify which subnets to use within the VPC.
     * Allows the user to select private, public, or isolated subnets.
     * @default All subnets in the VPC are used.
     */
    readonly subnetSelection?: SubnetSelection;
    /**
     * Custom commands to run during the install phase.
     *
     * **Example Usage:**
     * ```typescript
     * new TokenInjectableDockerBuilder(this, 'MyDockerBuilder', {
     *   path: path.resolve(__dirname, '../app'),
     *   installCommands: [
     *     'echo "Updating package lists..."',
     *     'apt-get update -y',
     *     'echo "Installing required packages..."',
     *     'apt-get install -y curl dnsutils',
     *   ],
     *   // ... other properties ...
     * });
     * ```
     * *This example demonstrates how to install the `curl` and `dnsutils` packages during the install phase using `apt-get`, the package manager for Ubuntu-based CodeBuild environments.*
     *
     * @default - No additional install commands.
     */
    readonly installCommands?: string[];
    /**
     * Custom commands to run during the pre_build phase.
     *
     * **Example Usage:**
     * ```typescript
     * new TokenInjectableDockerBuilder(this, 'MyDockerBuilder', {
     *   path: path.resolve(__dirname, '../app'),
     *   preBuildCommands: [
     *     'echo "Fetching configuration from private API..."',
     *     'curl -o config.json https://api.example.com/config',
     *   ],
     *   // ... other properties ...
     * });
     * ```
     * *In this example, the builder fetches a configuration file from a private API before starting the Docker build. This config file will be available in the same directory as your Dockerfile during CDK deployment.*
     *
     * @default - No additional pre-build commands.
     */
    readonly preBuildCommands?: string[];
}
/**
 * A CDK construct to build and push Docker images to an ECR repository using CodeBuild and Lambda custom resources.
 */
export declare class TokenInjectableDockerBuilder extends Construct {
    private readonly ecrRepository;
    readonly containerImage: ContainerImage;
    readonly dockerImageCode: DockerImageCode;
    constructor(scope: Construct, id: string, props: TokenInjectableDockerBuilderProps);
}
